#!/bin/bash

#set -x
#set -e

function usage() {
    echo "Usage: `basename $0` path kstart kincrement kend repetitions"
    exit 1
}

# Paths
toolpath=./../tools/bin
algpath=./../algorithms/bin

# Settings
repair="0"                   #fixes up files if they dont match dimension
factor="200"					       #factor which determines coreset- and chunksize

# better dont change something after this line unless you know what you do...

##################################################################################

# check for common mistakes
if [ -z "$1" -o ! -f "$1" -o $# -ne 5 ]; then
    usage
fi

if [ ! -d "$toolpath" ]; then
    echo "could not find directory $toolpath"
    exit 1
fi

if [ ! -d "$algpath" ]; then
    echo "could not find directory $algpath"
    exit 1
fi

# check for numerical values:
for i in `seq 2 1 5`; do
    p=`eval echo '$'$i`
    if [ -z "$p" ]; then
        echo "Param missing: $i"
		usage
        exit 1
    fi
    if ! [[ $p =~ ^[0-9]+$ ]]; then
        echo "Param error: $p"
		usage
        exit 1
    fi
done

# check for non-zero increments (would cause endless-loop)
if [ $3 -eq 0 ]; then
	echo "kinkremt should not be 0!"
	exit 1
fi


# extract working directory
wdir="`dirname $1`"

# extract centers
k=`seq $2 $3 $4`

# extract number of repetitions
r="$5"

# 1. validate CSV-file and transform to WSV-file
wsvpath="$1".wsv
foo="`\"${toolpath}/csv2wsv\" $repair \"$1\" \"${wsvpath}\"`"
if [ ! $? -eq 0 ]; then
  echo ${foo}
  exit 1
fi
n=${foo%% *}
d=${foo##$n }


# 2. create reportfile
date=`date`
datestamp=`date +"%F-Z%H:%M:%S"` 
report="${wdir}/report${datestamp}_`basename ${wsvpath}`.txt"

touch "$report"
cat > "$report" <<EOT
--- Clusteringreport ---

date: $date
call: `basename $0` $1 $2 $3 $4 $5 $6 $7 $8 $9

--- Fileinformation ---

input file: $1
number of points: $n
dimension: $d

EOT

# 5. Execute streamLS-algorithm and measure time
echo -e "\n----- streamLS -----" >> "$report"

for j in $k; do
  c=$(($j * $factor))
  echo "--- chunksize: $c" >> "$report"
  echo "-- expected number of centres: $j" >> "$report"
  echo "- runtime" >> "$report"

  for l in `seq 1 1 $r`; do
    seed=`"${toolpath}/mt"`
    echo "- random seed: $seed" >> "$report"
    echo "${seed}" > "seeds"
    echo "- runtime and memory:" >> "$report"

    /usr/bin/time -f "%U %M" "${algpath}/streamls" $j $j $n $d "${wsvpath}" 0 "${wdir}/localsearchcenters${c}_${j}_${l}.txt" $c > /dev/null 2>> "$report"
    echo "-------------------------------------------" >> "$report"
  done

  # calculate real number of centers
  echo "- real number of centres" >> "$report"
  for l in `seq 1 1 $r`; do
    foo=`wc -l "${wdir}/localsearchcenters${c}_${j}_${l}.txt"`
          realcenter=${foo%%"${wdir}/localsearchcenters${c}_${j}_${l}.txt"}           
    echo $realcenter >> "$report"
  done

  # calculate costs for streamLS
  echo "- costs" >> "$report"
  for l in `seq 1 1 $r`; do
    foo=`wc -l "${wdir}/localsearchcenters${c}_${j}_${l}.txt"`
    realcenter=${foo%%"${wdir}/localsearchcenters${c}_${j}_${l}.txt"}     
    ${toolpath}/clustercosts "$wsvpath" "${wdir}/localsearchcenters${c}_${j}_${l}.txt" $d $n $realcenter >> "$report"
  done
  echo -e "\n" >> "$report"
done
echo -e "\n" >> "$report"

# 9. finish reportfile
echo -e "\n-------------------------
This run was performed on $HOSTNAME by $USER." >> "$report"

exit 0

################## 
